/* File: vettord.c */
/* Time-stamp: "2001-02-19 02:53:04 calvanes" */
/* Scopo: funzioni per l'ordinamento di un vettore */


void SelectionSort(TipoVettore A, int n)
  /* Ordina i primi n elementi del vettore A usando l'algoritmo di ordinamento
     per selezione del minimo. */
{
  int i, j, i_min;
  TipoElemVettore temp;

  for (i = 0; i < n-1; i++) {
                                /* ricerca la componente minima in A[i..n-1] */
    i_min = i;
    for (j = i+1; j < n; j++)
      if (A[j] < A[i_min])
        i_min = j;

    if (i != i_min) {                             /* scambia A[i_min] e A[i] */
      temp = A[i_min];
      A[i_min] = A[i];
      A[i] = temp;
    }
  }
}  /* SelectionSort */


void BubbleSort(TipoVettore A, int n)
  /* Ordina i primi n elementi del vettore A usando l'algoritmo di ordinamento
     a bolle nella versione non ottimizzata. */
{
  int i, j;
  TipoElemVettore temp;

  for (i = 0; i < n-1; i++)                                 /* inizia fase i */
    for (j = n-1; j > i; j--)
      if (A[j] < A[j-1]) {                          /* scambia A[j] e A[j-1] */
        temp = A[j];
        A[j] = A[j-1];
        A[j-1] = temp;
      }
}  /* BubbleSort */


void BubbleSortOttimizzato(TipoVettore A, int n)
  /* Ordina i primi n elementi del vettore A usando l'algoritmo di ordinamento
     a bolle nella versione ottimizzata. */
{
  int i = 0;
  int j;
  TipoElemVettore temp;
  bool ordinato;

  do {                                                      /* inizia fase i */
    ordinato = TRUE;
    for (j = n-1; j > i; j--)
      if (A[j] < A[j-1]) {                          /* scambia A[j] e A[j-1] */
        temp = A[j];
        A[j] = A[j-1];
        A[j-1] = temp;
        ordinato = FALSE;
      }
    i++;
  } while (!ordinato && i < n-1);
}  /* BubbleSortOttimizzato */


void InsertionSort(TipoVettore A, int n)
  /* Ordina i primi n elementi del vettore A usando l'algoritmo di ordinamento
     per inserimento. */
{
  int elem_da_sistemare,        /* indice del prossimo elemento da sistemare */
      elem_da_controllare;            /* indice dell'elemento da controllare */
  TipoElemVettore valore_da_sistemare;  /* valore dell'elemento da sistemare */

  for (elem_da_sistemare = 0; elem_da_sistemare < n-1; elem_da_sistemare++) {
    valore_da_sistemare = A[elem_da_sistemare+1];

                                     /* controlla e scala gli elementi partendo
                                        dall'ultimo elemento sistemato */
    elem_da_controllare = elem_da_sistemare;
    while (elem_da_controllare >= 0 &&
           A[elem_da_controllare] > valore_da_sistemare) {
      A[elem_da_controllare+1] = A[elem_da_controllare];
      elem_da_controllare--;
    }
                /* sistema il valore da controllare nel posto rimasto libero */
    A[elem_da_controllare+1] = valore_da_sistemare;
  }
}  /* InsertionSort*/


void MergeVettore(TipoVettore A, int iniziale, int mediano, int finale)
  /* Fonde i due sottovettori ordinati di A da iniziale a mediano e
     da mediano+1 a finale in un unico sottovettore ordinato. */
{
  TipoVettore B;                                      /* vettore di appoggio */
  int primo, secondo, appoggio, da_copiare;

  primo = iniziale;
  secondo = mediano + 1;
  appoggio = iniziale;

  while (primo <= mediano && secondo <= finale) {
    if (A[primo] <= A[secondo]) {
      B[appoggio] = A[primo];
      primo++;
    }
    else {
      B[appoggio] = A[secondo];
      secondo++;
    }
    appoggio++;
  }

  if (secondo > finale)             /* e` finito prima il secondo sottovettore;
                                       copia da A in B tutti gli elementi del
                                       primo sottovettore fino a mediano */
    for (da_copiare = primo; da_copiare <= mediano; da_copiare++) {
      B[appoggio] = A[da_copiare];
      appoggio++;
    }
  else                              /* e` finito prima il primo sottovettore
                                       copia da A in B tutti gli elementi del
                                       secondo sottovettore fino a finale */
    for (da_copiare = secondo; da_copiare <= finale; da_copiare++) {
      B[appoggio] = A[da_copiare];
      appoggio++;
    }
                /* ricopia tutti gli elementi da iniziale a finale da B ad A */
  for (da_copiare = iniziale; da_copiare <= finale; da_copiare++)
    A[da_copiare] = B[da_copiare];
} /* MergeVettore */


void MergeRicorsivo(TipoVettore A, int iniziale, int finale)
  /* Ordina gli elementi del vettore A di indice compreso tra iniziale e finale
     usando l'algoritmo di ordinamento per fusione. */
{
  int mediano;

  if (iniziale < finale) {        /* l'intervallo da iniziale a finale, estremi
                                     inclusi comprende almeno due elementi */
    mediano = (iniziale + finale) / 2;

    MergeRicorsivo(A, iniziale, mediano);
    MergeRicorsivo(A, mediano+1, finale);

    MergeVettore(A, iniziale, mediano, finale);
  }
}  /* MergeRicorsivo */


void MergeSort(TipoVettore A, int n)
  /* Ordina i primi n elementi del vettore A usando l'algoritmo di ordinamento
     per fusione. */
{
  MergeRicorsivo(A, 0, n-1);
}  /* MergeSort */


void MergeVettoreOttimizzato(TipoVettore A, int iniziale, int mediano,
                             int finale)
  /* Fonde i due sottovettori ordinati di A da iniziale a mediano e
     da mediano+1 a finale in un unico sottovettore ordinato.
     Versione ottimizzata che evita di effettuare una doppia copia degli
     elementi finali di uno dei sottovettori.
  */
{
  TipoVettore B;   /* vettore di appoggio */
  int primo, secondo, appoggio, i;

  primo = iniziale;
  secondo = mediano + 1;
  appoggio = iniziale;

  while (primo <= mediano && secondo <= finale) {
    if (A[primo] <= A[secondo]) {
      B[appoggio] = A[primo];
      primo++;
    }
    else {
      B[appoggio] = A[secondo];
      secondo++;
    }
    appoggio++;
  }

  /* parte finale ottimizzata */

  if (secondo > finale)           /* e` finito prima il secondo sottovettore;
                                     copia da A in A stesso tutti gli elementi
                                     fino a mediano, incominciando dal basso */
    for (i = mediano; i >= primo; i--) {
      A[finale] = A[i];
      finale--;
    }

  /* NON c'e' il ramo else */

              /* copia tutti gli elementi da iniziale a appoggio-1 da B ad A */
  for (i = iniziale; i <= appoggio - 1; i++)
    A[i] = B[i];
}  /* MergeVettoreOttimizzato */


void MergeRicorsivoOttimizzato(TipoVettore A, int iniziale, int finale)
  /* Ordina gli elementi del vettore A di indice compreso tra iniziale e finale
     usando l'algoritmo di ordinamento per fusione.
     Utilizza la versione ottimizzata della funzione per effettuare il merge.
  */
{
  int mediano;

  int i;

  if (iniziale < finale) {
    mediano = (iniziale + finale) / 2;

    MergeRicorsivoOttimizzato(A, iniziale, mediano);
    MergeRicorsivoOttimizzato(A, mediano + 1, finale);

/*
    printf("merge %d-%d [ ", iniziale, mediano);
    for (i = iniziale; i <= mediano; i++)
      printf("%d ", A[i]);
    printf("] con %d-%d [ ", mediano + 1, finale);
    for (i = mediano + 1; i <= finale; i++)
      printf("%d ", A[i]);
    printf("] ==> [ ");
*/

    MergeVettoreOttimizzato(A, iniziale, mediano, finale);

/*
    for (i = iniziale; i<= finale; i++)
      printf("%d ", A[i]);
    printf("]\n");
*/
  }
}  /* MergeRicorsivo */


void MergeSortOttimizzato(TipoVettore A, int n)
  /* Ordina i primi n elementi del vettore A usando l'algoritmo di ordinamento
     per fusione.
     Utilizza la versione ottimizzata della funzione per effettuare il merge.
  */
{
  MergeRicorsivoOttimizzato(A, 0, n-1);
}  /* MergeSortOttimizzato */


void FusioneDueVettori(TipoVettore A, TipoVettore B, TipoVettore C, int n)
  /* Fonde i primi n elementi dei due vettori ordinati A e B nei primi 2n
     elementi del vettore C. */
{
  int i = 0, j = 0, k = 0;                          /* per scandire A, B e C */

  while (i < n && j < n) {
    if (A[i] <= B[j]) {
      C[k] = A[i];            /* 1 */
      i++;
    }
    else {
      C[k] = B[j];            /* 2 */
      j++;
    }
    k++;
  }
                       /* a questo punto uno dei due vettori A o B e' finito */
  if (i < n) {                /* si completa C con i rimanenti elementi di A */
    while (i < n) {
      C[k] = A[i];            /* 3 */
      i++;
      k++;
    }
  }
  else {                      /* si completa C con i rimanenti elementi di B */
    while (j < n) {
      C[k] = B[j];            /* 4 */
      j++;
      k++;
    }
  }
} /* FusioneDueVettori */
