/* File: vettrice.c */
/* Time-stamp: "2001-02-18 23:08:21 calvanes" */
/* Scopo: funzioni per la ricerca di un elemento in un vettore */


bool RicercaEsaustiva(TipoVettore A, TipoElemVettore elem, int n, int *posiz)
  /* Effettua la ricerca esaustiva di elem tra i primi n elementi di A.
     Il valore di ritorno e` TRUE se elem e` presente in A,
     FALSE altrimenti.  Se elem e` presente in A, posiz contiene l'indice della
     componente pari ad elem. */
{
  int i = -1;
  bool trovato = FALSE;

  do i++;
  while (i < n-1 && A[i] != elem );
  if (A[i] == elem) {
    trovato = TRUE;
    *posiz = i;
  }

  return trovato;
} /* RicercaEsaustiva */


bool RicercaBinariaRic(int inf, int sup, TipoVettore A, TipoElemVettore elem,
                       int *posiz)
  /* Funzione ricorsiva che effettua la ricerca di elem nella parte di A
     compresa fra inf e sup. */
{
  int med;
  bool trovato;

  if (inf > sup)
    trovato = FALSE;          /* la parte del vettore tra inf e sup e` vuota */
  else {
    med = (inf + sup) / 2;
    if (elem == A[med]) {                     /* l'elemento e` stato trovato */
      *posiz = med;
      trovato = TRUE;
    }
    else
      if (elem < A[med])                      /* cerca nella parte inferiore */
        trovato = RicercaBinariaRic(inf, med-1, A, elem, posiz);
      else                                    /* cerca nella parte superiore */
        trovato = RicercaBinariaRic(med+1, sup, A, elem, posiz);
  }

  return trovato;
}  /* RicercaBinariaRic */


bool RicercaBinaria(TipoVettore A, TipoElemVettore elem, int n, int *posiz)
  /* Effettua la ricerca binaria di elem tra i primi n elementi di A, usando la
     ricorsione.  Il valore di ritorno e` TRUE se elem e` presente in A, FALSE
     altrimenti.  Se elem e` presente in A, posiz contiene l'indice della
     componente pari ad elem. */
{
  return RicercaBinariaRic(0, n-1, A, elem, posiz);
}  /* RicercaBinaria */


bool RicercaBinariaIterativa(TipoVettore A, TipoElemVettore elem, int n,
                             int *posiz)
  /* Effettua la ricerca binaria di elem tra i primi n elementi di A, usando
     l'iterazione.  Il valore di ritorno e` TRUE se elem e` presente in A,
     FALSE altrimenti.  Se elem e` presente in A, posiz contiene l'indice della
     componente pari ad elem. */
{
  int inf, med, sup;
  bool trovato = FALSE;

  inf = 0;
  sup = n-1;

  while (inf <= sup && !trovato) {
    med = (inf + sup) / 2;
    if (elem == A[med]) {                     /* l'elemento e` stato trovato */
      *posiz = med;
      trovato = TRUE;
    }
    else
      if (elem < A[med])
        sup = med - 1;                        /* cerca nella parte inferiore */
      else
        inf = med + 1;                        /* cerca nella parte superiore */
  }

  return trovato;
}  /* RicercaBinariaIterativa */


bool EliminaDaVettore(TipoVettore A, TipoElemVettore elem, int *n, int *posiz)
  /* Elimina l'elemento elem tra i primi n elementi del vettore ordinato A.
     Il valore di ritorno e` TRUE se elem era presente in A ed e` stato
     eliminato, FALSE altrimenti.
     Se elem era presente in A, posiz contiene l'indice dell'elemento di A che
     era pari ad elem, ed n viene decrementato di uno.
     Utilizza la funzione RicercaBinaria. */
{
  int i;

  if (RicercaBinaria(A, elem, *n, posiz)) {
    (*n)--;
    for (i = *posiz;  i < *n; i++)
      A[i] = A[i+1];
  }
} /* EliminaDaVettore */


void RicercaMediano(TipoVettore A, int n, TipoElemVettore *mediano)
  /* Effettua la ricerca del mediano tra i primi n elementi del vettore A. */
{
  int i = -1;
  int j, maggiori, minori;
  bool trovato = FALSE;

  do {
    i++;
    minori = 0;
    maggiori = 0;
    for (j = 0; j < n; j++) {
      if (A[i] > A[j])
        minori++;
      if (A[i] < A[j])
        maggiori++;
    }
    if (minori == maggiori || minori + 1 == maggiori)
      trovato = TRUE;
  } while (!trovato);

  *mediano = A[i];
} /* RicercaMediano */
